@@ -1,6 +1,6 @@
 ##---------------------------------------------------------------------------##
 ##  File:
-##      $Id: mhopt.pl,v 2.53 2003/07/04 01:13:34 ehood Exp $
+##      $Id: mhopt.pl,v 2.56 2003/10/02 03:30:58 ehood Exp $
 ##  Author:
 ##      Earl Hood       mhonarc@mhonarc.org
 ##  Description:
@@ -46,6 +46,10 @@ sub get_resources {
 	'addressmodifycode=s',
 			# Perl expression for modifying displayed addresses
 	'annotate',	# Add a note to message(s)
+	'attachmentdir=s',
+			# Pathname to attachment files directory
+	'attachmenturl=s',
+			# URL to attachment files directory
 	'authsort',	# Sort by author
 	'archive',	# Create an archive (the default)
 	'conlen',	# Honor Content-Length fields
@@ -83,6 +87,7 @@ sub get_resources {
 	'idxprefix=s',	# Filename prefix for multi-page main index
 	'idxsize=i',	# Maximum number of messages shown in indexes
 	'keeponrmm',	# Do not delete message files on archive remove
+	'lang=s',	# Set locale/language
 	'localdatefmt=s',
 			# Date specification for local date
 	'lock',		# Do archive locking (default)
@@ -107,6 +112,7 @@ sub get_resources {
 	'msgprefix=s',	# Filename prefix for message files
 	'multipg',	# Generate multi-page indexes
 	'news',		# Add links to newsgroups
+	'newsurl=s',	# URL to use for news hyperlinks
 	'noauthsort',	# Do not sort by author
 	'noarchive',	# Do not create an archive
 	'nochecknoarchive',
@@ -291,6 +297,7 @@ sub get_resources {
     require 'mhtime.pl';
     require 'mhfile.pl';
     require 'mhutil.pl';
+    require 'mhrcfile.pl';
     require 'mhscan.pl'  	if $SCAN;
     require 'mhsingle.pl'  	if $SINGLE;
     require 'mhrmm.pl'  	if $RMM;
@@ -298,6 +305,9 @@ sub get_resources {
 
     print STDOUT "This is MHonArc v$VERSION, Perl $] $^O\n"  unless $QUIET;
 
+    ## Check for locale/lang setting
+    $Lang = $opt{'lang'}  if defined($opt{'lang'});
+
     ## Evaluate site local initialization
     delete($INC{'mhasiteinit.pl'});      # force re-evaluation
     eval { require 'mhasiteinit.pl'; };  # ignore status
@@ -305,24 +315,19 @@ sub get_resources {
     ## Read default resource file
     DEFRCFILE: {
 	if ($DefRcFile) {
-	    read_fmt_file($DefRcFile);
-	    last DEFRCFILE;
+	    last DEFRCFILE  if read_resource_file($DefRcFile);
 	}
-	if (defined $ENV{'HOME'}) {
+	my $home_dir = $ENV{'HOME'};
+	if (defined $home_dir) {
 	    # check if in home directory
-	    $tmp = join($DIRSEP, $ENV{'HOME'}, $DefRcName);
-	    if (-e $tmp) {
-		read_fmt_file($tmp);
-		last DEFRCFILE;
-	    }
+	    last DEFRCFILE
+		if read_resource_file(join($DIRSEP, $home_dir, $DefRcName), 1);
 	}
 	local $_;
 	foreach (@INC) {
-	    $tmp = join($DIRSEP, $_, $DefRcName);
-	    if (-e $tmp) {
-		read_fmt_file($tmp);
-		last DEFRCFILE;
-	    }
+	    next if ($_ eq $home_dir);
+	    last DEFRCFILE
+		if read_resource_file(join($DIRSEP, $_, $DefRcName), 1);
 	}
     }
 
@@ -429,12 +434,25 @@ sub get_resources {
 	$MHeadCnvFunc = \&readmail::MAILdecode_1522_str;
     }
 
-    ##	Read resource file(s) (I initially used the term 'format file').
+    ##	Read resource file(s)
     ##	Look for resource in outdir unless existing according to
     ##  current value.
     foreach (@FMTFILE) {
 	$_ = join($DIRSEP, $OUTDIR, $_) unless -e $_;
-	&read_fmt_file($_);
+	&read_resource_file($_);
+    }
+
+    ## Set locale
+    eval {
+	require POSIX;
+	if ($Lang) {
+	    POSIX::setlocal(&POSIX::LC_ALL, $Lang);
+	} else {
+	    POSIX::setlocal(&POSIX::LC_ALL, '');
+	}
+    };
+    if ($@ && $Lang) {
+	qq/Warning: Setting locale appears to be not supported:\n$@\n/;
     }
 
     ##	Re-check readmail settings
@@ -490,6 +508,7 @@ sub get_resources {
 	$IDXSIZE *= -1  if $IDXSIZE < 0;
     $OUTDIR	= $opt{'outdir'}     if $opt{'outdir'}; # Override db
     $MAILTOURL	= $opt{'mailtourl'}  if $opt{'mailtourl'};
+    $NewsUrl	= $opt{'newsurl'}    if $opt{'newsurl'};
     $MAXSIZE	= $opt{'maxsize'}    if defined($opt{'maxsize'});
 	$MAXSIZE = 0  if $MAXSIZE < 0;
     $MHPATTERN	= $opt{'mhpattern'}  if $opt{'mhpattern'};
@@ -526,6 +545,9 @@ sub get_resources {
 
     $IdxPageNum  = $opt{'pagenum'}   if defined($opt{'pagenum'});
 
+    $AttachmentDir = $opt{'attachmentdir'}  if defined($opt{'attachmentdir'});
+    $AttachmentUrl = $opt{'attachmenturl'}  if defined($opt{'attachmenturl'});
+
     ## Determine location of message note files
     $NoteDir = $opt{'notedir'}	if $opt{'notedir'};
 
@@ -759,15 +781,6 @@ sub usage {
     &mhusage();
 }
 
-##---------------------------------------------------------------------------
-##	read_fmt_file() requires the library with the resource file
-##	read subroutine and calls the routine.
-##
-sub read_fmt_file {
-    require 'mhrcfile.pl';
-    &read_resource_file;  # implicit passing of @_
-}
-
 ##---------------------------------------------------------------------------
 ##	Routine to update 1.x data structures to 2.0.
 ##
